/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package io.core.messaging;

import java.net.*;
import java.io.*;
import io.core.protocol.P2PPacket;
import salsa.language.ServiceFactory;

public class ObjectHandler extends Thread {
	private Socket incoming;
	private IOTheater theater = (IOTheater)ServiceFactory.getTheater();

	public ObjectHandler(Socket _incoming) {
		incoming = _incoming;
		setName( "Object Handler Thread" );
	}

	public void run() {
		// Read in the object.
		Object received = null;
		ObjectInputStream stream = null;

		while (incoming.isConnected()) {
			try {
				stream = new ObjectInputStream( incoming.getInputStream() );
				received = stream.readObject();
			} catch (ClassNotFoundException e) {
				System.err.println( "Could not load class for " + received.getClass().getName() );
				System.err.println( "  Is it in the theater CLASSPATH" );
				System.err.println( e.getMessage() );
				break;
			} catch (IOException e) {
				System.err.println( "IOException occured reading object " + e.getMessage() );
				break;
			}

			// Process the object.
			if (received instanceof P2PPacket) {
				theater.getProtocolHandler().process( (P2PPacket)received);
			} else {
				theater.getIOHandler().process(received, incoming);
			}

/*			try {
				stream.close();
			} catch (IOException e) {
				System.err.println("IOException occured closing stream: " + e.getMessage());
				break;
			}
*/
		}

		// Close the connection.
		try {
			incoming.close();
		} catch (IOException e) {
			System.err.println( "IOException occured closing connection" + e.getMessage() );

			return;
		}
	}
}
