/** 
	PeerHandler takes an incoming peer, and sends it a list
	of peers so that it can connect to the IO p2p network.

	Author: Travis Desell
	Last Modified: 9/20/2002
 */
package io.core.server;

import java.net.*;
import java.io.*;
import java.util.Vector;
import io.core.messaging.TheaterLocator;

public class PeerHandler extends Thread {
	// Must be a singleton so UniversalActor can get to it.
	private Socket incoming;
	private Vector peers;
	private int maxTheaters;

	public PeerHandler(Socket _incoming, Vector _peers, int _maxTheaters) {
		incoming = _incoming;
		peers = _peers;
		maxTheaters = _maxTheaters;

		setName( "Peer Handler Thread" );
	}

	/**
		This method takes the data of the incoming peer, and adds it to the
		hashtable of peers.  Then it sends a list of possible peers to 
		connect to.
	*/
	public void run() {
		Object received1 = null;
		Object received2 = null;

		int maxConnections;
		TheaterLocator peerTheater;

		//Read in the objects maxConnection and TheaterLocator
		try {
			ObjectInputStream stream = new ObjectInputStream( incoming.getInputStream() );
			received1 = stream.readObject();
			received2 = stream.readObject();
		} catch (ClassNotFoundException e) {
			System.err.println( "Could not load class for " + received1.getClass().getName() + " or " + received2.getClass().getName() );
			System.err.println( "  Is it in the theater CLASSPATH" );
			System.err.println( e.getMessage() );

			return;
		} catch (IOException e) {
			System.err.println( "IOException occured reading object " + e.getMessage() );

			return;
		}

		if ((received1 instanceof Integer) && (received2 instanceof TheaterLocator)) {
			maxConnections = ((Integer)received1).intValue();
			peerTheater = (TheaterLocator)received2;

			System.out.println("Recieved a request for " + maxConnections + " theaters by: ");
			System.out.println("\t" + peerTheater.toString() );
		}
		else {
			System.out.println("Received the wrong information from connected peerTheater");
			return;
		}

		// Close the connection.
		try {
			incoming.close();
		} catch (IOException e) {
			System.err.println( "IOException occured closing connection" + e.getMessage() );

			return;
		}

		//check and see if we already have this theater in our list
		boolean found = false;
		for (int i = 0; i < peers.size(); i++) {
			if ( ((TheaterLocator)peers.get(i)).equals(peerTheater) ) found = true;
		}
		if (!found && (peers.size() < maxTheaters) ) {
			peers.add(peerTheater);
		}
		SendPeers(maxConnections, peerTheater);
	}

	public void SendPeers(int maxConnections, TheaterLocator peerTheater) {
		//get a random peer to start at, and then read the maxConnection more peers

		if (peers.size() != 0) {
			int index = (int)java.lang.Math.round(java.lang.Math.random() * (peers.size() - 1));

			for (int i = 0; (i < maxConnections) && (i < peers.size()) ; i++) {
				TheaterLocator toSend = (TheaterLocator)peers.get(index);

				if (!peerTheater.equals(toSend)) {
					try {
						Socket remoteTheater = new Socket(peerTheater.getHost(), peerTheater.getPort());
						ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(remoteTheater.getOutputStream()));
	
						out.writeObject(toSend);
						out.flush();
						out.close();
						remoteTheater.close();
					} catch (UnknownHostException e) {
						System.err.println("Error sending information to server: " + toSend.toString() + ":"+e);
					} catch (IOException e) {
						System.err.println("Error sending information to server: " + toSend.toString() + ":"+e);
					}
				}

				index++;
				if (index >= peers.size()) index = 0;
			}
		}
	}
}
