/**
 * $Id: Actor.java,v 1.34 2003/03/25 04:35:50 stepha Exp $
 *
 * This is the main Actor class.
 */

package salsa.language;

import java.io.IOException; 
import salsa.language.exceptions.*; 
import salsa.messaging.*; 
import salsa.naming.*; 
import salsa.naming.exceptions.*;

public class Actor implements java.io.Serializable {
	protected UAN uan;

	public transient ActorState actorState;

	private transient MessagingService messagingService;
	private transient TheaterService theaterService;
	private transient NamingService namingService;

	public Actor() {
		uan = null;

		theaterService = ServiceFactory.getTheater();
		messagingService = ServiceFactory.getMessaging();
		namingService = ServiceFactory.getNaming();
	}



	/* The following cover local and remote message sending for Actors
	 */
	public void send(Message message) {
		
		message.messageSent = System.currentTimeMillis();
		
		if (isLocal()) {
			synchronized (getActorState().mailbox) {
				//local message send
				if (isLocal()) {
					getActorState().send(message);	
					return;
				}
			}
		}

		//remote message send
		messagingService.send(message, getUAL() );			
	}


	/* The following Functions provide basic access to protected and private members
	   of the Actor class
         */
	public ActorState getActorState() {
		if (uan != null) {
			Object target = theaterService.getEntry(uan);

			if (target instanceof ActorState) {
				return (ActorState)target;
			} else {
				return null;
			}
		} else {
			if (actorState != null) {
				return actorState;
			} else {
				System.err.println("ActorState and UAN are null, invalid Actor Reference: " + toString());
				return null;
			}
		}
	}


	/**
	 * These methods require the calling Actor to have the same type as
         * the actor corresponding to the UAN given.
	 * For examples, see migration.Agent, migration.Migrate and  
	 * migration.RemoteSend.
	 */     
	public void getReferenceByName(String uan) throws MalformedUANException {
		getReferenceByName(new UAN(uan)); 
	}
                
	public void getReferenceByName(UAN _uan) {  
		uan = _uan;
		UAL ual = namingService.resolve( uan );

		if (getActorState() == null) {
			theaterService.setEntry(uan, ual);
		}

		if (actorState != null) {
			actorState.stayAlive = false;
	                actorState = null;
		}
        }  
         

	public boolean isLocal() {
		if (uan != null) {
			return (theaterService.getEntry(uan) instanceof ActorState);
		} else if (actorState != null) {
			return true;
		} else {
			return false;
		}
	}

	public UAN getUAN() {
		return uan;
	}

	public UAL getUAL() {
		Object target = theaterService.getEntry(uan);

		if (target instanceof UAL) {
			return (UAL)target;
		} else {
			return null;
		}
	}

	public String toString() {
		return getClass().getName()+":"+super.toString();
	}

	public String getName() {
		return getActorState().getName();
	}



	/* Methods for Actor serializability
	*/

	private void writeObject(java.io.ObjectOutputStream out) throws SalsaException {
		if ((actorState != null) && (uan == null)) {
			uan = actorState.getUAN();
		}

		try {
			out.writeObject(uan);
			out.flush();
		} catch (IOException e) {
			throw new SalsaWrapperException( "Exception during actor reference serialization.", e);
		}
	}

	private void readObject(java.io.ObjectInputStream in) throws SalsaException {
		try {
			uan = (UAN)in.readObject();
		} catch (Exception e) {
			throw new SalsaWrapperException( "Exception during actor reference deserialization.", e);
		}

		//get new instances of the services used by the Actor Reference
		theaterService = ServiceFactory.getTheater();
		messagingService = ServiceFactory.getMessaging();
		namingService = ServiceFactory.getNaming();
	}
}
